function Centers = CMeans(Samples, InitialCenters, NormType, Criterion, Value)
//////////////////////////////////////////////////////////////////////////// 
// CLUSTER - Clustering Toolbox
//
// Copyright (c) by Dr. Eng. (J) Harald Galda, 2011
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
////////////////////////////////////////////////////////////////////////////


 // Input and output parameters are checked.
 
 [NumberOfOutputs NumberOfInputs] = argn();
 
 if NumberOfInputs ~= 5 then
 
  error('Wrong number of inputs.');
  
 end;
 
 // Samples must be a matrix.
 
 if (typeof(Samples) ~= 'constant') | (ndims(Samples) ~= 2)  then
  
  error('Samples must be a 2D matrix.');
  
 end;
 
 // InitialCenters must be a matrix.
 
 if (typeof(InitialCenters) ~= 'constant') | (ndims(InitialCenters) ~= 2)  then
  
  error('InitialCenters must be a 2D matrix.');
  
 end;
 
 // Samples and InitialCenters must have the same number of columns.
 
 if size(Samples, 2) ~= size(InitialCenters, 2) then
  
  error('Samples and InitialCenters must have the same number of columns');
  
 end;
 
 // NormType must be a scalar greater than zero.
 
 if (typeof(NormType) ~= 'constant') | (~isequal(size(NormType), [1 1])) then
 
  error('NormType must be a scalar greater than zero.');
 
 end;
 
 if NormType <= 0 then
  
  error('NormType must be a scalar greater than zero.');
  
 end;
 
 // Criterion must be a string and can have the values 'iterations' or 
 // 'quantization_error'. Criterion is not case sensitive.
 
 if (typeof(Criterion) ~= 'string') | (~isequal(size(Criterion), [1 1])) then
  
  error('Criterion must be a string and can be ''iterations'' or ' + ...
        '''quantization_error''.');
 
 end;

 // Value must be a scalar greater than or equal to zero.
 
 if (typeof(Value) ~= 'constant') | (~isequal(size(Value), [1 1])) then
 
  error('Value must be a scalar greater than or equal to zero.');
 
 end;
 
 if NumberOfOutputs ~= 1 then
  
  error('Wrong number of outputs.');
  
 end;
 
 // Initializations
 
 Centers = InitialCenters;

 NumberOfClusters = size(Centers, 1);

 IsCompleted = %f;

 // Main loop. For each row of Samples the nearest neighbor among the rows of
 // Centers is searched. If the termination condition is not fullfilled yet
 // (i. e. the specified number of iterations has not been achieved or the 
 // quantization error still exceeds the specified maximum), the cluster centers
 // are updated.
 
 while ~IsCompleted
  
  select convstr(Criterion, 'l')
  
   case 'iterations'
   
    if ~isdef('Iteration') then
     
     Iteration = 0;
     
    end;
    
    Iteration = Iteration + 1;
   
    IsCompleted = (Iteration > Value);
   
   case 'quantization_error'

    IsCompleted = (GetQuantizationError(Samples, Centers, NormType) <= Value);
   
  else
  
   error('Unknown termination criterion. Criterion can be ''iterations''' + ...
         'or ''quantization_error''.');
      
  end;
 
  if ~IsCompleted then

   [ClusterIndices Distances] = GetNearestNeighbor(Samples, Centers, NormType);
   
   for n = 1 : NumberOfClusters
   
    MemberIndices = find(ClusterIndices == n);
   
    if ~isempty(MemberIndices) then

     Centers(n, :) = mean(Samples(MemberIndices, :), 'r');

    end;
   
   end;

  end;
 
 end;

endfunction