function DaviesBouldinIndex = GetDaviesBouldinIndex(Samples, ...
                                                    Centers, ...
                                                    NormType, ...
                                                    CompactnessType)
//////////////////////////////////////////////////////////////////////////// 
// CLUSTER - Clustering Toolbox
//
// Copyright (c) by Dr. Eng. (J) Harald Galda, 2011
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
////////////////////////////////////////////////////////////////////////////


 // Input and output parameters are checked.
 
 [NumberOfOutputs NumberOfInputs] = argn();
 
 if (NumberOfInputs < 3) | (NumberOfInputs > 4) then
 
  error('Wrong number of inputs.');
  
 end;
 
  // Samples must be a matrix.
 
 if (typeof(Samples) ~= 'constant') | (ndims(Samples) ~= 2)  then
  
  error('Samples must be a 2D matrix.');
  
 end;
 
 // Centers must be a matrix.
 
 if (typeof(Centers) ~= 'constant') | (ndims(Centers) ~= 2)  then
  
  error('InitialCenters must be a 2D matrix.');
  
 end;
 
 // Samples and Centers must have the same number of columns.
 
 if size(Samples, 2) ~= size(Centers, 2) then
  
  error('Samples and InitialCenters must have the same number of columns');
  
 end;
 
 // There must be at least two distinct cluster centers.
 
 if size(unique(Centers, 'r'), 1) < 2 then
  
  error('There must be at least two distinc cluster centers');
  
 end;
 
 // NormType must be a scalar greater than zero.
 
 if (typeof(NormType) ~= 'constant') ...
 |  (~isequal(size(NormType), [1 1])) then
 
  error('NormType must be a scalar greater than zero.');
 
 end;
 
 if NormType <= 0 then
  
  error('NormType must be a scalar greater than zero.');
  
 end;

 if NumberOfInputs == 4 then

  // CompactnessType must be a scalar greater than zero.
   
  if (typeof(CompactnessType) ~= 'constant') ...
  |  (~isequal(size(CompactnessType), [1 1])) then
 
   error('CompactnessType must be a scalar greater than zero.');
 
  end;
 
  if CompactnessType <= 0 then
  
   error('CompactnessType must be a scalar greater than zero.');
  
  end;

 else
  
  CompactnessType = 1;

 end;

 // The distances between the cluster centers are calculated.
 
 CenterDistances = CreateDistanceMatrix(Centers, Centers, NormType);
 
 // For each cluster the compactness is calculated. 
 
 [Neighbors Distances] = GetNearestNeighbor(Samples, Centers, NormType);
 
 NumberOfClusters = size(Centers, 1);
 
 Compactnesses = zeros(NumberOfClusters, 1);
 
 for n = 1 : NumberOfClusters

  ListOfDistances = Distances(Neighbors == n, n);

  NumberOfMembers = length(ListOfDistances);
  
  if NumberOfMembers > 0 then

   Compactnesses(n) = norm(ListOfDistances / NumberOfMembers, CompactnessType);

  end;

 end;

 // For each pair of clusters (n, m) 
 // (Compactnesses(n) + Compactnesses(m)) / CenterDistances(n, m)
 // is calculated.
 
 Pairs = zeros(NumberOfClusters, NumberOfClusters);
 
 for n = 1 : NumberOfClusters

  for m = n + 1 : NumberOfClusters

   if CenterDistances(n, m) > 0 then

    PairValue = (Compactnesses(n) + Compactnesses(m)) / CenterDistances(n, m);

    Pairs(n, m) = PairValue;
   
    Pairs(m, n) = PairValue;

   end;

  end; 

 end;
 
 // The Davies-Bouldin index is calculated now.
 
 DaviesBouldinIndex = sum(max(Pairs, 'c')) / NumberOfClusters;
 
endfunction